<?php
/**
* One storyline. 
* An array of instances of this class, called $stripshow_story, constitutes the entire set of storylines.
* @package stripShow
*/
class Storyline {

	/**
	* This story's ID
	* @var int
	*/
	var $id;

	/**
	* The date on which this story starts.
	* @var string
	*/
	var $startdate;
	
	/**
	* The date on which this story ends.
	* Will be empty if this story is ongoing.
	* @var string
	*/
	var $enddate;
	
	/**
	* Name of the storyline.
	* @var string
	*/
	var $name;
	
	/**
	* The ID of this story's parent.
	* @var int
	*/
	var $parent;
	
	/**
	* This story's level.
	* Starts at 0.
	* @var int
	*/
	var $level;
	
	/**
	* Returns TRUE if storyline has children, FALSE if not.
	* @var bool
	*/
	var $has_children = FALSE;
	
	/**
	* An array of post objects that are in the story.
	* @var array
	*/
	var $parts = array();
	
	/**
	* An array of IDs of storylines that are children of the current one.
	* @var array
	*/
	var $children = array();
	
	/**
	* Finds children in the current story.
	* @uses $children
	* @uses $has_children
	* @uses $id
	*/
	function find_children( $stories ) {
		foreach( $stories as $x ) {
			if ( $x->parent == $this->id ) {
				array_push( $this->children, $x->id );
				$this->has_children = TRUE;
			}
		}
	}
	
	/**
	* Get the level of the current story.
	* Doesn't return anything. Instead, changes the $level var for this object.
	*/
	function get_level() {
		global $stripshow_story;
		if ( $this->parent > 0 ) {
			$a = $this->parent;
			$this->level = $stripshow_story[$a]->level + 1;
		}
		else $this->level = 0;
	}
		
	/**
	* Determine whether a given comic is within one of this story's children.
	* @param object $comic A comic object
	* @return bool Whether the comic is within one of the children of this story.
	*/
	function is_in_children( $comic ) {
		global $stripshow_story;
		foreach( $this->children as $child ) {
			if ( $stripshow_story[$child]->is_in_story( $comic ) ) return TRUE;
		}
		return FALSE;
	}
		
	/**
	* Determine whether a given comic is within this story.
	* @param object $comic A comic object
	* @return bool Whether the comic is a part of this story.
	* @uses is_in_children()
	* @uses $startdate
	* @uses $enddate
	* @uses $has_children
	*/
	function is_in_story_old( $comic ) {
		$date = $comic->post_date;
		if ( $this->startdate <= $date ) { 
			// This comic's date is after the story's start date.
			if ( $date < $this->enddate ) { 
				// This comic's date is within the story dates.
				if ( ! $this->has_children ) { 
					// This story has no children, so this comic is in the story.
					return TRUE;
				} else { 
					// This story does have children, so we need to figure out if this strip is in one of those children.
					if ( $this->is_in_children( $comic ) ) return FALSE; 
						// if this comic is in a child of this story, it's not in this story.
					else return TRUE;
				}
					return FALSE;
			} elseif ( ! isset( $this->enddate ) && $date <= date( 'Y-m-d H:i:s' ) ) { 
				// End date for this story isn't set, but the comic is before or during the actual date today.
				if ( $this->has_children ) { 
					// This story has children -- need to check the children.
					if ( $this->is_in_children( $comic ) ) return FALSE; 
						// if this comic is in a child of this story, it's not in this story.
					else return TRUE;
				} else { 
					// This story has no children.
					return TRUE;
				}
			} else { 
				// The comic is later than the actual date.
				return FALSE;
			}
	 	} else { 
			// Not after start date, so not in story.
			return FALSE;
		}
		// In theory, we should never get here. But if we do, return FALSE just in case.
		return false;
	}

	function is_in_story( $comic ) {
		global $stripshow_story;
		$date = $comic->post_date;
		if ( $this->startdate <= $date ) { // This comic's date is after the story's start date.
			if ( $date < $this->enddate ) { // This comic's date is within the story dates.
				if ( ! $this->has_children ) { // This story has no children, so this comic is in the story.
					return TRUE;
				} else { // This story does have children, so we need to figure out if this strip is in one of those children.
					foreach ( $this->children as $child ) {
						if ( $stripshow_story[$child]->is_in_story( $comic ) ) return FALSE;
					}
				}
					return FALSE;
			} elseif ( ! isset( $this->enddate ) && $date <= date( 'Y-m-d H:i:s' ) ) { // End date for this story isn't set, but the comic is before or during the actual date today.
				if ( $this->has_children ) { // This story has children -- need to check the children.
					if ( $this->is_in_children( $comic ) ) return FALSE; // if this comic is in a child of this story, it's not in this story.
					else {
						return TRUE;
					}
				} else { // This story has no children.
					return TRUE;
				}
			} else { // The comic is later than the actual date.
				return FALSE;
			}
		 } else { // Not after start date, so not in story.
			return FALSE;
		}
		// In theory, we should never get here. But if we do, return FALSE just in case.
		return false;
	}
		
	/**
	* Echoes the parts of this story in an unordered list
	*/
	function list_parts() {
		if ( empty( $this->parts ) ) return FALSE;
		echo '<ul class="storyline_parts">'."\n";
		foreach ( $this->parts as $part ) {
			echo '<li><a href="'.get_permalink( $part->ID ).'">'.$part->post_title."</a></li>\n";
		}
		echo "</ul>\n";
	}
	
	/**
	* Get the story's URL ( which is the URL of the first comic in the story ).
	* The "current story" should never be an empty story, since any calling
	* function will be from a member of some story.
	* @return string The URL
	* @uses $startdate
	*/
	function get_url() {
		global $wpdb, $stripShow;
		if ( ! empty( $this->parts ) ) return get_permalink( $this->parts[0] );
		else return FALSE;
	}
	
// End of StoryLine class.
}
